﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Security;
using DAMacTypesLib;

namespace DAMac {
    [SuppressUnmanagedCodeSecurityAttribute]
    internal static class SafeNativeMethods {
        internal static uint MAKELONG(ushort wLow, ushort wHigh) { return (uint)((wLow & 0xffff) | (wHigh & 0xffff) << 16); }
        internal static byte LOBYTE(ushort wValue) { return (byte)(wValue & 0xff); }
        internal static byte HIBYTE(ushort wValue) { return (byte)(wValue >> 8); }

        [Flags] //MMSystem.h
        internal enum SoundPlayerFlgs : uint {
            SND_ASYNC = 0x0001,
            SND_FILENAME = 0x00020000
        }
        //WinUser.h
        internal enum WindowMessage : uint {
            WM_KEYDOWN = 0x0100,
            WM_KEYUP = 0x0101,
            WM_CHAR = 0x0102,
            WM_SYSKEYDOWN = 0x0104,
            WM_SYSKEYUP = 0x0105,
            WM_MOUSEMOVE = 0x0200,
            WM_LBUTTONDOWN = 0x0201,
            WM_LBUTTONUP = 0x0202,
            WM_RBUTTONDOWN = 0x0204,
            WM_RBUTTONUP = 0x0205,
            WM_MBUTTONDOWN = 0x0207,
            WM_MBUTTONUP = 0x0208
        }

        internal enum WindowMessageSetPtr : int {
            GWL_EXSTYLE = -20,
            WS_EX_TOOLWINDOW = 0x00000080,
            WS_EX_APPWINDOW = 0x00040000
        }

        /*ATOM GlobalAddAtom(
            LPCTSTR lpString
        );*/
        [DllImport("kernel32", CharSet = CharSet.Unicode)]
        internal static extern ushort GlobalAddAtom(string lpString);
        /*ATOM GlobalDeleteAtom(
            ATOM nAtom
        );*/
        [DllImport("kernel32")]
        internal static extern ushort GlobalDeleteAtom(ushort nAtom);
        /*BOOL PostMessage(
            HWND daWin,
            UINT Msg,
            WPARAM wParam,
            LPARAM lParam
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool PostMessage(IntPtr hWnd, WindowMessage msg, UIntPtr wParam, UIntPtr lParam);
        /*UINT MapVirtualKey(
            UINT uCode,
            UINT uMapType  ((uMapType 0 = VK->SC, 1 SC->VK))
        );*/
        [DllImport("user32.dll")]
        internal static extern uint MapVirtualKey(uint uCode, uint uMapType);
        /*BOOL RegisterHotKey(
            HWND hWnd,
            int id,
            UINT fsModifiers,
            UINT vk
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool RegisterHotKey(IntPtr hWnd, int id, WindowModifiers fsModifiers, uint vk);
        /*BOOL UnregisterHotKey(
            HWND hWnd,
            int id
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool UnregisterHotKey(IntPtr hWnd, int id);
        /*BOOL ScreenToClient(
            HWND hWnd,
            LPPOINT lpPoint
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool ScreenToClient(IntPtr hWnd, ref Point lpPoint);
        /*BOOL ClientToScreen(
            HWND hWnd,
            LPPOINT lpPoint
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool ClientToScreen(IntPtr hWnd, ref Point lpPoint);
        /*HWND GetForegroundWindow(VOID);*/
        [DllImport("user32.dll")]
        internal static extern IntPtr GetForegroundWindow();
        /*BOOL WINAPI AttachThreadInput(
            DWORD idAttach,
            DWORD idAttachTo,
            BOOL fAttach
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool AttachThreadInput(uint idAttach, uint idAttachTo, [MarshalAs(UnmanagedType.Bool)] bool fAttach);
        /*DWORD GetWindowThreadProcessId(
            HWND hWnd,
            LPDWORD lpdwProcessId
        );*/
        [DllImport("user32.dll")]
        internal static extern uint GetWindowThreadProcessId(IntPtr hWnd, out uint lpdwProcessId);
        /*BOOL GetKeyboardState(
            PBYTE lpKeyState
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool GetKeyboardState(byte[] lpKeyState);
        /*BOOL SetKeyboardState(
            LPBYTE lpKeyState
        );*/
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool SetKeyboardState(byte[] lpKeyState);
        /*SHORT VkKeyScan(
            TCHAR ch
        );*/
        [DllImport("user32.dll")]
        internal static extern short VkKeyScan(char ch);
        /*LONG_PTR GetWindowLongPtr(
            HWND hWnd,
            int nIndex
        );*/
        internal static IntPtr MyGetWindowLongPtr(IntPtr hWnd, int nIndex) { return IntPtr.Size == 8 ? GetWindowLongPtr(hWnd, nIndex) : new IntPtr(GetWindowLong(hWnd, nIndex)); }
        [DllImport("user32.dll")]
        internal static extern int GetWindowLong(IntPtr hWnd, int nIndex);
        [DllImport("user32.dll")]
        internal static extern IntPtr GetWindowLongPtr(IntPtr hWnd, int nIndex);
        /*LONG_PTR SetWindowLongPtr(
            HWND hWnd,
            int nIndex,
            LONG_PTR dwNewLong
        );*/
        internal static IntPtr MySetWindowLongPtr(IntPtr hWnd, int nIndex, IntPtr dwNewLong) { return IntPtr.Size == 8 ? SetWindowLongPtr(hWnd, nIndex, dwNewLong) : new IntPtr(SetWindowLong(hWnd, nIndex, (int)dwNewLong)); }
        [DllImport("user32.dll")]
        internal static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);
        [DllImport("user32.dll")]
        internal static extern IntPtr SetWindowLongPtr(IntPtr hWnd, int nIndex, IntPtr dwNewLong);
    }
}